$ifndef _LOL_Shield_Inc_
$define _LOL_Shield_Inc_
'
' Code to operate the Arduino LOL shield
' Written by Les Johnson for the Proton BASIC compiler, operating the LOL shield on the Amicus18 board
'
    #disable Print                                  ' Remove the compiler's Print library subroutine
    Include "Amicus18_Timers.inc"                   ' Include the Amicus18 Timer macros into the program

    On_Hardware_Interrupt GoTo ISR_Routine          ' Point the interrupt to its handler subroutine

    Declare LCD_Type = Graphic                      ' Inform the compiler that we're using a graphic display
    Declare LCD_X_Res = 14
    Declare LCD_Y_Res = 9
'
' Create mask values for access to the LED matrix
'
$define cMsk0 1
$define cMsk1 2
$define cMsk2 4
$define cMsk3 8
$define cMsk4 16
$define cMsk5 32
$define cMsk6 64
$define cMsk7 128

$define cLOL_Max_X 13
$define cLOL_Max_Y 8
'
' Variables used within the interrupt to matrix scan the display
'
    Dim LED_bIndex As Byte System
    Dim LED_dPortPin As Dword System
    Dim LED_dPrevPortPin As Dword System
'
' Routine Variables
'
    Dim LED_bColour As Byte System                  ' Used to hold the colour of the pixel for all routines except DisplayChar
    Dim LED_bPixelX As Byte System                  ' Used by the Plot/Pixel subroutines to hold the X position of the pixel
    Dim LED_bPixelY As Byte System                  ' Used by the Plot/Pixel subroutines to hold the Y position of the pixel
    Dim LED_bAddress As Byte System                 ' Used by the Plot/Pixel subroutines to calculate the pixel address
'
' Declare variables used by DisplayChar and DisplayString
'
    Dim LOL_bCharX As GEN
    Dim LOL_bCharY As GENH
    Dim LOL_bCol As Byte
    Dim LOL_bRow As Byte
    Dim LOL_bChar As Byte
    Dim LOL_LED_bColSave As Byte
    Dim LOL_wCharOffset As PRODL.Word               ' Alias the SFR pair PRODL and PRODH as a word variable
    Dim LOL_wSaveTblptr As Word
'
' Declare variables (aliases) used by LOL_Line
'
    Dim LOL_bDeltaX As GEN
    Dim LOL_bDeltaY As GENH
    Dim LOL_bStepX As Byte
    Dim LOL_bStepY As Byte
    Dim LOL_bDeviation As Byte
    Dim LOL_bYpos_S As PP1
    Dim LOL_bXpos_S As PP1H
    Dim LOL_bYpos_E As PP2
    Dim LOL_bXpos_E As PP2H
'
' Declare variables used by LOL_Circle
'
    Dim LOL_bXX As GEN
    Dim LOL_bYY As GENH
    Dim LOL_wTR As SWord
    Dim LOL_bDD As SByte
    Dim LOL_bQuadA As PP2
    Dim LOL_bQuadB As PP2H
    Dim LOL_bQuadC As PP3
    Dim LOL_bQuadD As GEN2
    Dim LOL_bRadius As GEN2H

    Dim LED_bDisplayBuffer[126] As Byte Heap            ' Holds the LED matrix information

    Dim LOL_wFSR0 As FSR0L.Word                         ' Combine FSR0L\H into a 16-bit register
    Dim LOL_wFSR1 As FSR1L.Word                         ' Combine FSR1L\H into a 16-bit register
    Dim LOL_wTblPtr As TBLPTRL.Word                     ' Combine TBLPTRL\H into a 16-bit register
    Dim LOL_wCCPR2 As CCPR2L.Word                       ' Combine CCPR2L\H into a 16-bit register
'
' Compiler system variables. Re-used by the routines
'
    Dim GEN As Byte System
    Dim GENH As Byte System
    Dim GEN2 As Byte System
    Dim GEN2H As Byte System
    Dim PP0 As Byte System
    Dim PP0H As Byte System
    Dim PP1 As Byte System
    Dim PP1H As Byte System
    Dim PP2 As Byte System
    Dim PP2H As Byte System
    Dim PP3 As Byte System
    Dim PP3H As Byte System
'
' Create a table in flash memory of the pins required to charlieplex the display matrix
'
'                                       Line 1
'
    Dim lDisplayMap As Code = As Byte LATB,cMsk2, LATC,cMsk2,  LATB,cMsk2, LATC,cMsk1,  LATB,cMsk2, LATC,cMsk0,  LATB,cMsk2, LATB,cMsk7,_
                                      LATB,cMsk2, LATB,cMsk6,  LATB,cMsk2, LATB,cMsk5,  LATB,cMsk2, LATB,cMsk4,  LATB,cMsk2, LATB,cMsk3,_
                                      LATB,cMsk2, LATC,cMsk3,  LATC,cMsk3, LATB,cMsk2,  LATB,cMsk2, LATC,cMsk4,  LATC,cMsk4, LATB,cMsk2,_
                                      LATB,cMsk2, LATC,cMsk5,  LATC,cMsk5, LATB,cMsk2,_
                                      '
                                      ' Line 2
                                      '
                                      LATB,cMsk3, LATC,cMsk2,  LATB,cMsk3, LATC,cMsk1,  LATB,cMsk3, LATC,cMsk0,  LATB,cMsk3, LATB,cMsk7,_
                                      LATB,cMsk3, LATB,cMsk6,  LATB,cMsk3, LATB,cMsk5,  LATB,cMsk3, LATB,cMsk4,  LATB,cMsk3, LATB,cMsk2,_
                                      LATB,cMsk3, LATC,cMsk3,  LATC,cMsk3, LATB,cMsk3,  LATB,cMsk3, LATC,cMsk4,  LATC,cMsk4, LATB,cMsk3,_
                                      LATB,cMsk3, LATC,cMsk5,  LATC,cMsk5, LATB,cMsk3,_
                                      '
                                      ' Line 3
                                      '
                                      LATB,cMsk4, LATC,cMsk2,  LATB,cMsk4, LATC,cMsk1,  LATB,cMsk4, LATC,cMsk0,  LATB,cMsk4, LATB,cMsk7,_
                                      LATB,cMsk4, LATB,cMsk6,  LATB,cMsk4, LATB,cMsk5,  LATB,cMsk4, LATB,cMsk3,  LATB,cMsk4, LATB,cMsk2,_
                                      LATB,cMsk4, LATC,cMsk3,  LATC,cMsk3, LATB,cMsk4,  LATB,cMsk4, LATC,cMsk4,  LATC,cMsk4, LATB,cMsk4,_
                                      LATB,cMsk4, LATC,cMsk5,  LATC,cMsk5, LATB,cMsk4,_
                                      '
                                      ' Line 4
                                      '
                                      LATB,cMsk5, LATC,cMsk2,  LATB,cMsk5, LATC,cMsk1,  LATB,cMsk5, LATC,cMsk0,  LATB,cMsk5, LATB,cMsk7,_
                                      LATB,cMsk5, LATB,cMsk6,  LATB,cMsk5, LATB,cMsk4,  LATB,cMsk5, LATB,cMsk3,  LATB,cMsk5, LATB,cMsk2,_
                                      LATB,cMsk5, LATC,cMsk3,  LATC,cMsk3, LATB,cMsk5,  LATB,cMsk5, LATC,cMsk4,  LATC,cMsk4, LATB,cMsk5,_
                                      LATB,cMsk5, LATC,cMsk5,  LATC,cMsk5, LATB,cMsk5,_
                                      '
                                      ' Line 5
                                      '
                                      LATB,cMsk6, LATC,cMsk2,  LATB,cMsk6, LATC,cMsk1,  LATB,cMsk6, LATC,cMsk0,  LATB,cMsk6, LATB,cMsk7,_
                                      LATB,cMsk6, LATB,cMsk5,  LATB,cMsk6, LATB,cMsk4,  LATB,cMsk6, LATB,cMsk3,  LATB,cMsk6, LATB,cMsk2,_
                                      LATB,cMsk6, LATC,cMsk3,  LATC,cMsk3, LATB,cMsk6,  LATB,cMsk6, LATC,cMsk4,  LATC,cMsk4, LATB,cMsk6,_
                                      LATB,cMsk6, LATC,cMsk5,  LATC,cMsk5, LATB,cMsk6,_
                                      '
                                      ' Line 6
                                      '
                                      LATB,cMsk7, LATC,cMsk2,  LATB,cMsk7, LATC,cMsk1,  LATB,cMsk7, LATC,cMsk0,  LATB,cMsk7, LATB,cMsk6,_
                                      LATB,cMsk7, LATB,cMsk5,  LATB,cMsk7, LATB,cMsk4,  LATB,cMsk7, LATB,cMsk3,  LATB,cMsk7, LATB,cMsk2,_
                                      LATB,cMsk7, LATC,cMsk3,  LATC,cMsk3, LATB,cMsk7,  LATB,cMsk7, LATC,cMsk4,  LATC,cMsk4, LATB,cMsk7,_
                                      LATB,cMsk7, LATC,cMsk5,  LATC,cMsk5, LATB,cMsk7,_
                                      '
                                      ' Line 7
                                      '
                                      LATC,cMsk0, LATC,cMsk2,  LATC,cMsk0, LATC,cMsk1,  LATC,cMsk0, LATB,cMsk7,  LATC,cMsk0, LATB,cMsk6,_
                                      LATC,cMsk0, LATB,cMsk5,  LATC,cMsk0, LATB,cMsk4,  LATC,cMsk0, LATB,cMsk3,  LATC,cMsk0, LATB,cMsk2,_
                                      LATC,cMsk0, LATC,cMsk3,  LATC,cMsk3, LATC,cMsk0,  LATC,cMsk0, LATC,cMsk4,  LATC,cMsk4, LATC,cMsk0,_
                                      LATC,cMsk0, LATC,cMsk5,  LATC,cMsk5, LATC,cMsk0,_
                                      '
                                      ' Line 8
                                      '
                                      LATC,cMsk1, LATC,cMsk2,  LATC,cMsk1, LATC,cMsk0,  LATC,cMsk1, LATB,cMsk7,  LATC,cMsk1, LATB,cMsk6,_
                                      LATC,cMsk1, LATB,cMsk5,  LATC,cMsk1, LATB,cMsk4,  LATC,cMsk1, LATB,cMsk3,  LATC,cMsk1, LATB,cMsk2,_
                                      LATC,cMsk1, LATC,cMsk3,  LATC,cMsk3, LATC,cMsk1,  LATC,cMsk1, LATC,cMsk4,  LATC,cMsk4, LATC,cMsk1,_
                                      LATC,cMsk1, LATC,cMsk5,  LATC,cMsk5, LATC,cMsk1,_
                                      '
                                      ' Line 9
                                      '
                                      LATC,cMsk2, LATC,cMsk1,  LATC,cMsk2, LATC,cMsk0,  LATC,cMsk2, LATB,cMsk7,  LATC,cMsk2, LATB,cMsk6,_
                                      LATC,cMsk2, LATB,cMsk5,  LATC,cMsk2, LATB,cMsk4,  LATC,cMsk2, LATB,cMsk3,  LATC,cMsk2, LATB,cMsk2,_
                                      LATC,cMsk2, LATC,cMsk3,  LATC,cMsk3, LATC,cMsk2,  LATC,cMsk2, LATC,cMsk4,  LATC,cMsk4, LATC,cMsk2,_
                                      LATC,cMsk2, LATC,cMsk5,  LATC,cMsk5, LATC,cMsk2

'--------------------------------------------------------------------------------------------
    GoTo _LOLShield_Main_                                   ' Jump over the subroutines
'--------------------------------------------------------------------------------------------
' Change the colour of the pen when displaying
' Input     : pColour holds the colour of the LED. i.e. On or Off
' Output    : None
' Notes     : None
'
$define LOL_Pen(pColour) LED_bColour = pColour

'--------------------------------------------------------------------------------------------
' Special Even Interrupt Handler
' Illuminates the LEDs into a charlieplexed matrix
' Input     : None
' Output    : None
' Notes     : None
'
ISR_Routine:
    Context Save TBLPTRL, TBLPTRH, TABLAT, FSR1L, FSR1H
    If PIR2bits_CCP2IF = 1 Then                             ' Was it a Compare Special Event on CCP2 that triggered the interrupt?
        FSR1H = $0F                                         ' Point to LATB, LATC, TRISB, or TRISC
        '
        ' Make both the previous pins inputs
        '
        FSR1L = LED_dPrevPortPin.Byte0                      ' Place the previous low byte of the high port into FSR1L
        FSR1L = FSR1L + $09                                 ' Point to the TRIS
        WREG = INDF1 | LED_dPrevPortPin.Byte1               ' \ Set the high pin to 1 (input)
        INDF1 = WREG                                        ' /

        FSR1L = LED_dPrevPortPin.Byte2                      ' Place the previous low byte of the low port into FSR1L
        FSR1L = FSR1L + $09                                 ' Point to the TRIS
        WREG = INDF1 | LED_dPrevPortPin.Byte3               ' \ Set the low pin to 1 (input)
        INDF1 = WREG                                        ' /

        WREG = LED_bDisplayBuffer[LED_bIndex]               ' Read an element within the display buffer RAM
        If WREG <> 0 Then                                   ' Does it contain a value other than 0?
            '
            ' Read the high and low LED ports.bits from flash memory
            '
            LOL_wTblPtr = LED_bIndex
            LOL_wTblPtr = LOL_wTblPtr * 4
            Movlw(lDisplayMap & 255)
            Addwf TBLPTRL,F,0
            Movlw((lDisplayMap >> 8) & 255)
            Addwfc TBLPTRH,F,0
            Tblrd*+
            LED_dPortPin.Byte0 = TABLAT
            Tblrd*+
            LED_dPortPin.Byte1 = TABLAT
            Tblrd*+
            LED_dPortPin.Byte2 = TABLAT
            Tblrd*+
            LED_dPortPin.Byte3 = TABLAT
            '
            ' Set the Port.Pin high
            '
            FSR1L = LED_dPortPin.Byte0                      ' Place the low byte of the high port into FSR1L
            WREG = INDF1 | LED_dPortPin.Byte1               ' \ Set the pin to 1 (high)
            INDF1 = WREG                                    ' /

            FSR1L = FSR1L + $09                             ' Point to the TRIS of the high port
            WREG = LED_dPortPin.Byte1                       ' \
            WREG = WREG ^ 255                               ' | Set pin to 0 (Output)
            INDF1 = INDF1 & WREG                            ' /
            '
            ' Set the Port.Pin low
            '
            FSR1L = LED_dPortPin.Byte2                      ' Place the low byte of the low port into FSR1L
            WREG = LED_dPortPin.Byte3                       ' \
            WREG = WREG ^ 255                               ' | Set the pin to 0 (Low)
            INDF1 = INDF1 & WREG                            ' /

            FSR1L = FSR1L + $09                             ' Point to the TRIS of the low port
            WREG = LED_dPortPin.Byte3                       ' \
            WREG = WREG ^ 255                               ' | Set pin to 0 (Output)
            INDF1 = INDF1 & WREG                            ' /
        EndIf
        LED_dPrevPortPin = LED_dPortPin                     ' Keep a copy of the previous Port.Pin that was accessed
        Inc LED_bIndex
        If LED_bIndex = 126 Then                            ' Have we reached the last LED?
            LED_bIndex = 0                                  ' Yes. So go back to the start one
        EndIf
        PIR2bits_CCP2IF = 0                                 ' Clear the interrupt flag
    EndIf
'
' << More Interrupt Conditions and Code can be placed here >>
'
    Context Restore                                         ' Exit the interrupt

'---------------------------------------------------------------------------------------------------
' Set or clear a display LED
' Input     : pXpos holds the X position of the LED
'           : pYpos holds the Y position of the LED
'           : pColour holds 0 if the LED is to be cleared
' Output    : None
' Notes     : Plot a point in the display RAM
'
$define LOL_Plot(pXpos, pYpos, pColour) '
    LED_bPixelX = pXpos                 '
    LED_bColour = pColour               '
    WREG = pYpos                        '
    GoSub _LOL_Plot

_LOL_Plot:
    LED_bPixelY = WREG                                  ' Transfer WREG (LED_bPixelY) into its working variable
    If LED_bPixelY.7 = 1 Then Return                    ' \
    If LED_bPixelY > cLOL_Max_Y Then Return             ' |
    If LED_bPixelX.7 = 1 Then Return                    ' | Boundary checks
    If LED_bPixelX > cLOL_Max_X Then Return             ' /

    LED_bAddress = LED_bPixelY * 14                     ' \ Calculate where in the display RAM we need for X and Y
    LED_bAddress = LED_bAddress + LED_bPixelX           ' /
    If LED_bColour = 0 Then
        LED_bDisplayBuffer[LED_bAddress] = 0            ' Place the new value into the RAM
    Else
        LED_bDisplayBuffer[LED_bAddress] = 1
    EndIf
    Return

'---------------------------------------------------------------------------------------------------
' Read the display and return if the LED is illuminated or not
' Input     : pXpos holds the X position of the LED
'           : pYpos holds the Y position of the LED
' Output    : pResult holds 0 If the LED is celar, otherwise 1
' Notes     : None
'
$define LOL_Pixel(pXpos, pYpos, pResult) '
    LED_bPixelX = pXpos                  '
    WREG = pYpos                         '
    _LOL_Pixel                           '
    pResult = WREG

_LOL_Pixel Macro
    GoSub __LOL_Pixel
Endm

#ifMacro- _LOL_Pixel
__LOL_Pixel:
    LED_bPixelY = WREG                                  ' Transfer WREG (LED_bPixelY) into its working variable
    If LED_bPixelY.7 = 1 Then _LOL_Pixel_Error          ' \
    If LED_bPixelY > cLOL_Max_Y Then _LOL_Pixel_Error   ' |
    If LED_bPixelX.7 = 1 Then _LOL_Pixel_Error          ' | Boundary checks
    If LED_bPixelX > cLOL_Max_X Then _LOL_Pixel_Error   ' /

    LED_bAddress = LED_bPixelY * 14                     ' \ Calculate where in the display RAM we need for X and Y
    LED_bAddress = LED_bAddress + LED_bPixelX           ' /
    WREG = LED_bDisplayBuffer[LED_bAddress]             ' Read from the correct location within display RAM
    Return
_LOL_Pixel_Error:
    WREG = 0
    Return
#endIfMacro-

'---------------------------------------------------------------------------------------------------
' Draw or Erase a line from pXStart, pYStart to pXEnd, pYEnd
' Input     : pXStart = Starting X position of the Line
'           : pYStart = Starting Y position of the line
'           : pXEnd = Ending X position of the line
'           : pYEnd = Ending Y position of the line
'           : pColour = Colour of the LED. i.e. On or Off
' Output    : None
' Notes     : None
'
$define LOL_Line(pXStart, pYStart, pXEnd, pYEnd, pColour) '
    LOL_bXpos_S = pXStart                        '
    LOL_bYpos_S = pYStart                        '
    LOL_bXpos_E = pXEnd                          '
    LED_bColour = pColour                        '
    WREG = pYEnd                                 '
    _LOL_Line

_LOL_Line Macro
    GoSub __LOL_DrawLine
Endm

#ifMacro- _LOL_Line
Declare Warnings = Off
Declare Small_Micro_Model = On
__LOL_DrawLine:
    LOL_bYpos_E = WREG                                          ' Transfer WREG into it's working variable
    LOL_bDeltaY = LOL_bYpos_E - LOL_bYpos_S                     ' Find the initial Y direction
    LOL_bDeltaX = LOL_bXpos_E - LOL_bXpos_S                     ' Find the initial X direction

    LOL_bStepY = 1                                              ' Default to a positive step for Ypos
    If LOL_bDeltaY.7 = 1 Then                                   ' Check if negative
        LOL_bDeltaY = -LOL_bDeltaY                              ' Negate LOL_bDeltaY
        LOL_bStepY = -1                                         ' And indicate a negative step
    EndIf
    LOL_bStepX = 1                                              ' Default to a positive step for Xpos
    If LOL_bDeltaX.7 = 1 Then                                   ' Check if negative
        LOL_bDeltaX = -LOL_bDeltaX                              ' Negate LOL_bDeltaX
        LOL_bStepX = -1                                         ' And indicate a negative step
    EndIf
    LOL_Plot(LOL_bXpos_S,LOL_bYpos_S, LED_bColour)             ' The first LED plot is a special case
    If LOL_bDeltaX > LOL_bDeltaY Then
        LOL_bDeviation = LOL_bDeltaY - LOL_bDeltaX
        While LOL_bXpos_S <> LOL_bXpos_E
            If LOL_bDeviation.7 = 0 Then                        ' Check if negative
                LOL_bYpos_S = LOL_bYpos_S + LOL_bStepY          ' Move on the Y axis directed by LOL_bStepY
                LOL_bDeviation = LOL_bDeviation - LOL_bDeltaX
            EndIf
            LOL_bXpos_S = LOL_bXpos_S + LOL_bStepX              ' Move on the X axis directed by LOL_bStepX
            LOL_bDeviation = LOL_bDeviation + LOL_bDeltaY
            LOL_Plot(LOL_bXpos_S,LOL_bYpos_S, LED_bColour)     ' Illuminate or Extinguish the LED at this position
        Wend
        Return                                                  ' Return prematurely (this saves an else command)
    EndIf
    LOL_bDeviation = LOL_bDeltaX - LOL_bDeltaY

    While LOL_bYpos_S <> LOL_bYpos_E
        If LOL_bDeviation.7 = 0 Then                            ' Check if negative
            LOL_bXpos_S = LOL_bXpos_S + LOL_bStepX              ' Move on the X axis directed by LOL_bStepX
            LOL_bDeviation = LOL_bDeviation - LOL_bDeltaY
        EndIf
        LOL_bYpos_S = LOL_bYpos_S + LOL_bStepY                  ' Move on the Y axis directed by LOL_bStepY
        LOL_bDeviation = LOL_bDeviation + LOL_bDeltaX
        LOL_Plot(LOL_bXpos_S,LOL_bYpos_S, LED_bColour)         ' Illuminate or Extinguish the LED at this position
    Wend
    Return
Declare Small_Micro_Model = Off
#endIfMacro-

'-----------------------------------------------------------------------------------------
' Draw a circle to the LED matrix
' Input     : pXpos holds th X position
'           : pYpos holds the Y position
'           : pRadius holds the radius of the circle
'           : pColour = Colour of the LED. i.e. On or Off
' Output    : None
' Notes     : None
'
$define LOL_Circle(pXpos, pYpos, pRadius, pColour) '
    LOL_bXpos_S = pXpos                            '
    LOL_bYpos_S = pYpos                            '
    LED_bColour = pColour                          '
    WREG = pRadius                                 '
    _LOL_Circle

_LOL_Circle Macro
    GoSub __LOL_Circle
Endm

#ifMacro- _LOL_Circle
__LOL_Circle:
    LOL_bRadius = WREG                                  ' Transfer WREG into its working variable
    If LOL_bRadius = 0 Then Inc LOL_bRadius             ' Trap a zero radius
    LOL_bDD = LOL_bYpos_S - LOL_bXpos_S
    LOL_bXX = 0
    LOL_bYY = LOL_bRadius
    LOL_wTR = 3 - (2 * LOL_bRadius)
    While LOL_bXX <= LOL_bYY
        LOL_bQuadA = LOL_bXpos_S + LOL_bXX
        LOL_bQuadB = LOL_bYpos_S + LOL_bYY
        LOL_bQuadC = LOL_bXpos_S - LOL_bXX
        LOL_bQuadD = LOL_bYpos_S - LOL_bYY
        GoSub LOL_PlotIt
        LOL_bQuadA = LOL_bYpos_S + LOL_bYY - LOL_bDD
        LOL_bQuadB = LOL_bYpos_S + LOL_bXX
        LOL_bQuadC = LOL_bYpos_S - LOL_bYY - LOL_bDD
        LOL_bQuadD = LOL_bYpos_S - LOL_bXX
        GoSub LOL_PlotIt
        If LOL_wTR < 0 Then
            LOL_wTR = LOL_wTR + 6
            LOL_wTR = LOL_wTR + (4 * LOL_bXX)
        Else
            LOL_wTR = LOL_wTR + 10
            LOL_wTR = LOL_wTR + (4 * (LOL_bXX - LOL_bYY))
            Dec LOL_bYY
        EndIf
        Inc LOL_bXX
    Wend
    Return
'
' Plot 4 locations simultaneously
' This is part of the Circle subroutine
'
LOL_PlotIt:
    LOL_Plot(LOL_bQuadA, LOL_bQuadB, LED_bColour)
    LOL_Plot(LOL_bQuadA, LOL_bQuadD, LED_bColour)
    LOL_Plot(LOL_bQuadC, LOL_bQuadB, LED_bColour)
    LOL_Plot(LOL_bQuadC, LOL_bQuadD, LED_bColour)
    Return
#endIfMacro-

'---------------------------------------------------------------------------------------------------
'Display a character on teh LED matrix
' Input     : pChar holds the character to display
' Output    :WREG still holds the character displayed
'Notes     : UsesLOL_Plottodraweachdot of the character
'
$define LOL_DisplayChar(pChar) '
    WREG = pChar               '
    _LOL_DisplayChar

_LOL_DisplayChar Macro
    GoSub __LOL_DisplayChar
Endm

#ifSym __SYSCOM_GLCD_PRINT_REQ_
@Print
__LOL_DisplayChar:
    PP3H = WREG                                         ' Transfer WREG into its working variable
    LOL_wSaveTblptr = LOL_wTblPtr                       ' Save the contents of TBLPTRL\H
'
'Movecharacterdefinition,LEDbyLED,ontothedisplay
' Fontsaredefinedasonebytepercolumn
'
' Characters's Address = PP3H * 6
' PP3H holds the byte to display
' Add the offset for the address of font: LOL_wCharOffset + font label's address
' This is the code that requires altering for different sized fonts
'
    LOL_bCol = 0
    Repeat                                              ' Create a loop for all the bytes in the character's font
        LOL_wCharOffset = PP3H * 6                      ' 6 bytes per character font
        LOL_wCharOffset = LOL_wCharOffset + LOL_bCol    ' Add the offset to the character's value
        TABLAT = CRead8 Font_Table[LOL_wCharOffset]     ' Read the byte from the font table
        '
        ' Send the value in TABLAT to the Display
        '
        LOL_bRow = 7
        Repeat
            LED_bColour = TABLAT.6
            LOL_Plot(LOL_bCharX + LOL_bCol,LOL_bCharY + LOL_bRow, LED_bColour)
            TABLAT = TABLAT << 1
            Dec LOL_bRow
        Until LOL_bRow = 0
        Inc LOL_bCol                                    ' Next byte in the font
    Until LOL_bCol >= 5                                 ' Have we done 5 writes yet?
    LOL_wTblPtr = LOL_wSaveTblptr                       ' Yes. So Restore the value of TBLPTRL/H
    LOL_bCharX = LOL_bCharX + 6                         ' Move 6 LEDs to the right (with each character)
    WREG = PP3H                                         ' Restore the value of WREG
    Return
#endIfSym

'---------------------------------------------------------------------------------------------------
' Set the position of a character
' Input     : pXpos holds the X position
'           : pYpos holds the Y position
' Output    : None
' Notes     : None
'
$define LOL_Cursor(pXpos, pYpos) '
    LOL_bCharX = pXpos           '
    LOL_bCharY = pYpos

'---------------------------------------------------------------------------------------------------
' Clear the display
' Input     : pColour is 0 if the display is to be cleared, otherwise 1 will fill it
' Output    : None
' Notes     : None
'
$define LOL_Cls(pColour)         '
    If pColour = 0 Then          '
        Clear LED_bDisplayBuffer '
    Else                         '
        Set LED_bDisplayBuffer   '
    EndIf                        '
    LOL_bCharX = 0               '
    LOL_bCharY = 0

'---------------------------------------------------------------------------------------------------
' Scroll the LED matrix Down by one line
' Input     : None
' Output    : None
' Notes     : None
'
$define LOL_ScrollDown() _LOL_ScrollDown

_LOL_ScrollDown Macro-
    GoSub __LOL_ScrollDown
Endm

#ifMacro- _LOL_ScrollDown
__LOL_ScrollDown:
    LOL_bDeviation = 125                        ' Start at the end of the display RAM array
    LOL_bDeltaX = cLOL_Max_X                    ' Start at the end of a line of LEDs
    Repeat                                      ' Create a loop for the row of LEDs
        LOL_bDeltaY = 9                         ' \
        Repeat                                  ' / Create a loop the column of LEDs
            LED_bDisplayBuffer[LOL_bDeviation] = LED_bDisplayBuffer[LOL_bDeviation - 14] ' Transfer the previous LEDs into the current LEDs
            Dec LOL_bDeviation
            Dec LOL_bDeltaY
        Until LOL_bDeltaY = 0
        '
        ' Clear the top line of LEDs
        '
        LOL_bDeltaY = 0
        Repeat
            LED_bDisplayBuffer[LOL_bDeltaY] = 0
            Inc LOL_bDeltaY
        Until LOL_bDeltaY = 14
        Dec LOL_bDeltaX
    Until LOL_bDeltaX = 0
    Return
#endIfMacro-

'---------------------------------------------------------------------------------------------------
' Scroll the LED matrix Up by one line
' Input     : None
' Output    : None
' Notes     : None
'
$define LOL_ScrollUp() _LOL_ScrollUp

_LOL_ScrollUp Macro-
    GoSub __LOL_ScrollUp
Endm

#ifMacro- _LOL_ScrollUp
__LOL_ScrollUp:
    LOL_bDeviation = 0
    LOL_bDeltaX = cLOL_Max_X
    Repeat
        LOL_bDeltaY = 9
        Repeat
            LED_bDisplayBuffer[LOL_bDeviation] = LED_bDisplayBuffer[LOL_bDeviation + 14]
            Inc LOL_bDeviation
            Dec LOL_bDeltaY
        Until LOL_bDeltaY = 0
        '
        ' Clear the bottom line of LEDs
        '
        LOL_bDeltaY = 125
        Repeat
            LED_bDisplayBuffer[LOL_bDeltaY] = 0
            Dec LOL_bDeltaY
        Until LOL_bDeltaY = 112
        Dec LOL_bDeltaX
    Until LOL_bDeltaX = 0
    Return
#endIfMacro-

'---------------------------------------------------------------------------------------------------
' Scroll the LED matrix Right by one line
' Input     : None
' Output    : None
' Notes     : None
'
$define LOL_ScrollRight() _LOL_ScrollRight

_LOL_ScrollRight Macro-
    GoSub __LOL_ScrollRight
Endm

#ifMacro- _LOL_ScrollRight
__LOL_ScrollRight:
    LOL_bDeviation = 125 '0
    LOL_bDeltaY = 9
    Repeat
        LOL_bDeltaX = cLOL_Max_X
        Repeat
            LED_bDisplayBuffer[LOL_bDeviation] = LED_bDisplayBuffer[LOL_bDeviation - 1]
            Dec LOL_bDeviation
            Dec LOL_bDeltaX
        Until LOL_bDeltaX = 0
        LED_bDisplayBuffer[LOL_bDeviation] = 0
        Dec LOL_bDeviation
        Dec LOL_bDeltaY
    Until LOL_bDeltaY = 0
    Return
#endIfMacro-

'---------------------------------------------------------------------------------------------------
' Scroll the LED matrix Left by one line
' Input     : None
' Output    : None
' Notes     : None
'
$define LOL_ScrollLeft() _LOL_ScrollLeft

_LOL_ScrollLeft Macro-
    GoSub __LOL_ScrollLeft
Endm

#ifMacro- _LOL_ScrollLeft
__LOL_ScrollLeft:
    LOL_bDeviation = 0
    LOL_bDeltaY = 9
    Repeat
        LOL_bDeltaX = cLOL_Max_X
        Repeat
            LED_bDisplayBuffer[LOL_bDeviation] = LED_bDisplayBuffer[LOL_bDeviation + 1]
            Inc LOL_bDeviation
            Dec LOL_bDeltaX
        Until LOL_bDeltaX = 0
        LED_bDisplayBuffer[LOL_bDeviation] = 0
        Inc LOL_bDeviation
        Dec LOL_bDeltaY
    Until LOL_bDeltaY = 0
    Return
#endIfMacro-

'---------------------------------------------------------------------------------------------------
' Setup the program
' Input     : None
' Output    : None
' Notes     : None
'
$define LOL_Setup() GoSub _LOL_Setup

_LOL_Setup:
    LOL_bCharX = 0
    LOL_bCharY = 0
    LED_bIndex = 0                                          ' Reset the LED index used within the interrupt
    LED_dPrevPortPin = CRead32 lDisplayMap[0]               ' Make a copy of the current Port.Pin that is to be accessed
    Clear LED_bDisplayBuffer

    CCP2CON = %00001011                                     ' Compare mode: trigger special event, reset timer, start A/D conversion on CCP2 match (tCCP2IF bit is set)

    OpenTimer1(TIMER_INT_OFF & T1_16BIT_RW & T1_PS_1_8 & T1_SYNC_EXT_OFF & T1_SOURCE_INT & T1_SOURCE_CCP)
    $define cPrescalerValue 8                               ' Alter this to match the prescaler parameter above. i.e. 4 for T3_PS_1_4, 8 for T3_PS_1_8
    $define cMicroSeconds 150                               ' Interrupt rate (in uS)
'
' Calculate the value to place into the CCPRx register in order to achieve a certain interrupt rate (in us)
'
    $define cCCPR_Value $eval (cMicroSeconds / cPrescalerValue) * (_xtal / 4)

    LOL_wCCPR2 = cCCPR_Value                                ' Load CCPR2L\H with the value to trigger an interrupt at a certain duration
    PIE2bits_CCP2IE = 1                                     ' Enable the Special Event Interrupt on CCP2
    INTCONbits_PEIE = 1                                     ' Enable Peripheral Interrupts
    INTCONbits_GIE = 1                                      ' Enable Global Interrupts
    Return

'---------------------------------------------------------------------------------------------------
_LOLShield_Main_:

    LOL_Setup()                                             ' Setup the matrix interface
    LOL_Cls(0)                                              ' Clear the display
    LOL_Pen(1)                                              ' Set the default colour to on

'#ifSym __SYSCOM_GLCD_PRINT_REQ_
'    Include "Font.inc"
'#endIfSym
$endif

