'
' Code to operate the Arduino LOL shield from an Amicus18 board
' Produces a type of Plasma effect on the LED matrix
'
' Written by Les Johnson for the Proton BASIC compiler, operating the LOL shield on the Amicus18 board
'
    Include "Amicus18.inc"                      ' Configure the compiler for the PIC18F25K20 at 64MHz. i.e. An Amicus18 board
    Include "LOLShield.inc"                     ' Load the LOL Shield routines into the program
'
' Create some variables for the demo
'
    Dim bRow As Byte
    Dim bCol As Byte
    Dim fRow As Float
    Dim fCol As Float
    Dim fDistance As Float
    Dim fColour As Float
    Dim fPhase As Float

    Dim Dist1_Xpos As Float
    Dim Dist1_Ypos As Float
    Dim Dist2_Xpos As Float
    Dim Dist2_Ypos As Float

    Dim Point1_Xpos As Float
    Dim Point1_Ypos As Float
    Dim Point2_Xpos As Float
    Dim Point2_Ypos As Float

'-------------------------------------------------------------------------------------------------------
' The main program loop starts here
'
Main:
    fPhase = 0
    While
        fPhase = fPhase + 0.08                    ' Controls the speed of the moving points. Higher = faster.
        '
        ' The two points move along Lissajious curves, see: http:'en.wikipedia.org/wiki/Lissajous_curve
        ' We want values that fit the LED grid: X values between 0..13 and Y values between 0..8
        '
        Point1_Xpos = Sin(fPhase * 1.000)
        Point1_Xpos = Point1_Xpos + 1.0
        Point1_Xpos = Point1_Xpos * 7.5

        Point1_Ypos = Sin(fPhase * 1.310)
        Point1_Ypos = Point1_Ypos + 1.0
        Point1_Ypos = Point1_Ypos * 4.0

        Point2_Xpos = Sin(fPhase * 1.770)
        Point2_Xpos = Point2_Xpos + 1.0
        Point2_Xpos = Point2_Xpos * 7.5

        Point2_Ypos = Sin(fPhase * 2.865)
        Point2_Ypos = Point2_Ypos + 1.0
        Point2_Ypos = Point2_Ypos * 4.0
        '
        ' For each row...
        '
        bRow = 9
        Repeat
            Dec bRow
            fRow = bRow
            '
            ' For each column...
            '
            bCol = 14
            Repeat
                Dec bCol
                fCol = bCol
                '
                ' Calculate the Distance between this LED and Point1
                '
                Dist1_Xpos = fCol - Point1_Xpos         ' \
                Dist1_Ypos = fRow - Point1_Ypos         ' / The vector from Point1 to this LED
                fDistance = (Dist1_Xpos * Dist1_Xpos) + (Dist1_Ypos * Dist1_Ypos)
                '
                ' Calculate the distance between this LED and Point2
                '
                Dist2_Xpos = fCol - Point2_Xpos         ' \
                Dist2_Ypos = fRow - Point2_Ypos         ' / The vector from Point2 to this LED
                '
                ' Multiply this with the other distance, this will create plasma values
                '
                fDistance = fDistance * ((Dist2_Xpos * Dist2_Xpos) + (Dist2_Ypos * Dist2_Ypos))
                fDistance = Sqr(fDistance)

                fColour = Sin(fDistance * 0.1)
                fColour = fColour + 1.0

                LOL_Plot(bCol, bRow, fColour)           ' Illuminate or extinguish an LED
            Until bCol = 0
        Until bRow = 0
        DelayMS 5                                       ' SLow down so we can see the LEDs moving because Proton's code is too efficient
    Wend


