'
' Code to operate the Arduino LOL shield
' Create a Scrolling message accross the display
'
' Written by Les Johnson for the Proton BASIC compiler, operating the LOL shield on the Amicus18 board
'
'
    Include "Amicus18.inc"                      ' Configure the compiler for the PIC18F25K20 at 64MHz. i.e. An Amicus18 board
    Include "LOLShield.inc"                     ' Load the LOL Shield routines into the program

$define cScrollTime 200                         ' The time between scrolls happening
'
' Create some variables for the demo
'
    Dim bChar As Byte                           ' Holds the character to display
    Dim wCharOffset As Word                     ' Holds the font address
    Dim bColumn As Byte
    Dim bRow As Byte
'
' Create the flash memory message to be scrolled accross the LED matrix
'
    Dim MyCodeString[100] As Code = "LOLShield Operating on the Amicus18 Board Written In Proton BASIC.."

'---------------------------------------------------------------------------------------------------
    GoTo Main                                           ' Jump over the subroutines

'---------------------------------------------------------------------------------------------------
'Copyacharacterglyphfromthefontdatatabletodisplaymemory
' Input     : pChar holds the character to display
' Output    :None
'Notes     : UsesLOL_Plottodraweachdot of the character
'
$define ScrollDisplayChar(pChar) '
    bChar = pChar                '
    GoSub _ScrollDisplayChar

_ScrollDisplayChar:
    LOL_wSaveTblptr = LOL_wTblPtr                       ' Save the contents of TBLPTRL\H
'
'Movecharacterdefinition,LEDbyLED,ontothedisplay
' Fontsaredefinedasonebytepercolumn
'
' Characters's Address = bChar * 6
' PP3H holds the byte to display
' Add the offset for the address of font: wCharOffset + font label's address
' This is the code that requires altering for different sized fonts
'
    bColumn = 0
    Repeat                                              ' Create a loop for all the bytes in the character's font
        wCharOffset = bChar * 6                         ' 6 bytes per character font
        wCharOffset = wCharOffset + bColumn             ' Add the offset to the character's value
        TABLAT = CRead8 Font_Table[wCharOffset]         ' Read the byte from the font table
        '
        ' Send the value in TABLAT to the Display
        '
        bRow = 7
        Repeat
            LED_bColour = TABLAT.6
            LOL_Plot(13,bRow, LED_bColour)
            TABLAT = TABLAT << 1
            Dec bRow
        Until bRow = 0
        LOL_ScrollLeft()                                ' Scroll the matrix left one line
        DelayMS cScrollTime                             ' Slow down so we can see the scroll happening
        Inc bColumn                                     ' Next byte in the font
    Until bColumn >= 5                                  ' Have we done 5 writes yet?
    LOL_ScrollLeft()                                    ' Yes. So scroll the matrix left one line
    DelayMS cScrollTime                                 ' Slow down so we can see the scroll happening
    LOL_wTblPtr = LOL_wSaveTblptr                       ' Restore the contents of TBLPTRL\H
    Return

'---------------------------------------------------------------------------------------------------
' Display a RAM string of characters on the LED matrix
' Input     : pStr holds the RAM string to display
' Output    : None
' Notes     : SFR pair FSR1L\H hold the address of the RAM String To display
'
$define DisplayString(pStr) '
    LOL_wFSR1 = AddressOf(pStr) '
    GoSub _DisplayString

_DisplayString:
    While                                               ' Create a loop
        bChar = POSTINC1                                ' Transfer the element of the string into bChar and increment position
        If bChar = 0 Then Break                         ' Exit when a null found
        ScrollDisplayChar(bChar)                        ' Display the character held in bChar
    Wend
    Return

'---------------------------------------------------------------------------------------------------
' Display a Code Memory string of characters on the LED matrix
' Input     : pStr holds the code memory characters to display
' Output    : None
' Notes     : SFR pair TBLPTRL\H hold the address of the code memory String to display
'
$define DisplayCodeString(pStr)   '
    LOL_wTblPtr = AddressOf(pStr) '
    GoSub _DisplayCodeString

_DisplayCodeString:
    While                                               ' Create a loop
        Tblrd*+                                         ' Read from flash memory and increment the address (into TABLAT)
        bChar = TABLAT                                  ' Transfer the character from the string into bChar
        If bChar = 0 Then Break                         ' Exit when a null found
        ScrollDisplayChar(bChar)                        ' Display the character held in bChar
    Wend
    Return
'---------------------------------------------------------------------------------------------------
Main:
    LOL_Cls(0)                                          ' Clear the LED matrix
    LOL_Pen(1)                                          ' Set the characters to display

    While                                               ' Create an infinite loop
        DisplayCodeString(MyCodeString)                 ' Display the message with scrolling
    Wend                                                ' Do it forever

'---------------------------------------------------------------------------------
' Load the font used for the characters
'
    Include "Font.inc"

